function [y, grad] = shEvalGrad( l, m, theta, phi)
%
% [y, grad] = shEvalGrad( l, m, theta, phi)
%
% Evaluate the (l,m)-th real Spherical Harmonic basis function and its 
% gradient at (theta,phi). The gradient is equivalent to the second 
% Vector Spherical Harmonic at (theta,phi) (see the books by Freeden for 
% more details on vector Vpherical Harmonics).
% 
% @note (theta,phi) can be arrays
%
% @return y       (l,m)-th Spherical Harmonics value at (theta,phi)
% @return grad    ( dy_lm / d\theta , dy_lm / d\phi) |_{\theta,\phi}
% @param  l       Spherical Harmonic band
% @param  m       Harmonic within band l
% @param  theta   theta value(s) where to evaluate the gradient
% @param  phi     phi  value(s) where to evaluate the gradient

  % Schmidt's normalized Legendre polynomial avoids that we have to deal
  % with the factorial terms in the normalization constant
  [p_nm, dp_nm] = legendre2( l, cos(theta), 'sch');
  % extract the m value of interest
  p_nm = p_nm(abs(m)+1,:)';
  dp_nm = dp_nm(abs(m)+1,:)';

  % note: legendre2 returns the derivatives w.r.t. cos(theta), that is 
  % including the factor -sin(theta) resulting from the chain rule
  
  if( 0 == m)
    y = p_nm;
    grad(:,1) = dp_nm;
    grad(:,2) = 0;
  elseif (m > 0)
    y = cos(m * phi) .* p_nm;
    grad(:,1) = dp_nm .* cos(m * phi);
    grad(:,2) = -m * sin(m * phi) .* p_nm;
  else
    y = sin(abs(m) * phi) .* p_nm;
    grad(:,1) = dp_nm .* sin(abs(m) * phi);
    grad(:,2) = abs(m) * cos(abs(m) * phi) .* p_nm;
  end

  sh_normalize = sqrt((2*l + 1) / (4*pi));
  y = y' .* sh_normalize;
  grad = grad' .* sh_normalize;

end
