function [f_l, grad_l] = shEvalGradFunctionL( coeffs, band, omega) 
%
% [f_l, grad_l] = shEvalGradFunctionL( coeffs, band, omega) 
%
% Evaluate the l-th Spherical Harmonics band and the gradient of the 
% function f represented by \a coeffs at the point omega = (theta,phi). 
% The function f can have different channels which have to correspond to 
% different rows of \a coeffs.
%
% @return f_l     function value of f represented by coeffs restricted to
%                 the l-th Spherical Harmonics band
% @return grad_l  gradient of the f_l
% @param  coeffs  Spherical Harmonics representation of the function f to
%                 evaluate.
% @param  band    Spherical Harmonics band to evaluate.
% @param  omega   Point on S^2 where to evaluate f.

  % multiple data channels are currently not supported
  assert( 1 == size(coeffs,1));

  f_l = zeros( size( coeffs, 1), size( omega,1));
  grad_l = zeros( size( coeffs, 1), size( omega,1), 2);

  for( m = -band : band)
      
    % linear index into the coefficient vector
    index = band * (band+1) + m + 1;
    
    [y,grad] = shEvalGrad( band, m, omega(:,1), omega(:,2)); 
    
    f_l = f_l + coeffs(:,index) * y;
    for( i = 1 : size(coeffs(1)))
      grad_l(i,:,:) = ...
            transpose(squeeze(grad_l(i,:,:))' + coeffs(i,index) * grad);
    end
    
    
  end

end

