function coeffs_band_r = shRotateRKHSSpectralL( band, coeffs, R )
%
% coeffs_band_r = shRotateRKHSSpectralL( band, coeffs, R)
%
% Rotate sub-band of a Spherical Harmonics signal given by coeffs 
% by beta around the Y axis.
%
% @return coeffs_band_r  SH coefficients of the rotated sub-band signal
% @param band            Sub-band to rotate
% @param coeffs          SH coefficient of the (entire) unrotated signal
% @param R               rotation

  % latitude for sample points
  zeta = deg_to_rad(85); 

  % number of samples needed for this band
  num_samples = 2*band + 1;

  % generate sample points
  sample_pts = uniformPointsS2( num_samples, 'semi-equator', zeta);
    
  % rotate sample points
  sample_pts_r = rotatePointsS2( sample_pts, R');

  % evaluate sub-band signal at rotated sample points
  f_l = shEvalFunctionL( band, coeffs, sample_pts_r);

  % perform FFT of f_l to obtain the Harmonic components

  h_l = fft(f_l);
  h_l_sin = imag(h_l);
  h_l_cos = real(h_l);

  % map harmonic coefficients to Spherical Harmonics coefficients

  % the associated Legendre polynomials evaluated at the chosen latitude are
  % needed to recover the coefficients
  
  p_lm = legendre( band, cos(zeta), 'sch');
  p_lm = p_lm * 1/2 * sqrt((2*band+1) / (4*pi)) * num_samples;
  
  coeffs_sin = -1.0 * h_l_sin(1:band+1) ./ p_lm';
  coeffs_sin = coeffs_sin(end:-1:2);
  coeffs_cos = h_l_cos(1:band+1) ./ p_lm';
  coeffs_cos(1) = coeffs_cos(1) * 1/2;  

  coeffs_band_r = [coeffs_sin coeffs_cos];
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function coeffs_band_r = fixInaccuracies( band, coeffs, coeffs_band_r, ...
                                          idx, zeta_2, ...
                                          num_samples, R )
% Attempt to fix inaccurate coefficients
% We handle at the moment at most one almost vanishing Legendre
% coefficient. Extension to multiple coefficients are possible but we
% believe that other algorithms are more appropriate if precision is of
% high importance and / or high-band signals are to be considered.
%
% Note that the function is not optimized but intended to demonstrate the
% principle of how to fix inaccurate coefficients

  % create additional sample points
  delta = (2*pi) / num_samples;
  phi = delta / 2;
  omicron = [];
  for( i = 1 : numel(idx))
    omicron = [ omicron ; ...
                zeta_2,  phi ; ...
                zeta_2, -phi ];
    phi = phi + delta;
  end

  % rotate additional sample pionts
  omicron_r = rotatePointsS2( omicron, R');

  % full function values at the rotated sample points
  f_l_omicron = shEvalFunctionL( coeffs, band+1, omicron_r);

  % reconstruct partial signal assuming the inaccurate coefficients are
  % zero
  coeffs_band_r((band+1)-(idx-1)) = 0;
  coeffs_band_r((band+1)+(idx-1)) = 0;

  y_omicron = [];
  for( i = 1 : size( omicron,1))
    for( m = -band : band) 
      y_omicron(m+band+1, i) = shEval( band, m, ...
                                       omicron(i,1), omicron(i,2));
    end
  end

  % compute partially reconstructed signal
  f_l_omicron_p = [];
  for( i = 1 : size(omicron,1))
    f_l_omicron_p(i) = dot( y_omicron(:,i)', coeffs_band_r);
  end

  % compute difference signal
  d_f_l = f_l_omicron - f_l_omicron_p;

  % set up linear system
  Y = zeros(size(omicron,1));
  for( i = 1 : size(omicron,1))
    for( j = 1 : numel(idx))
      Y(i,2*j-1) = shEval( band, -(idx(j)-1), omicron(i,1),omicron(i,2));
      Y(i,2*j)   = shEval( band,  (idx(j)-1), omicron(i,1),omicron(i,2));
    end
  end

  % solve linear system for remaining coefficients
  b = Y \ d_f_l';

  % replace inaccurate coefficients
  for( j = 1 : numel(idx))
    coeffs_band_r((band+1)-(idx(j)-1)) = b(2*j-1);
    coeffs_band_r((band+1)+(idx(j)-1)) = b(2*j);
  end
    
end
