% Example for SH rotation using reproducing kernel Hilbert spaces

clear;

%% Setup and precomputation

% include necessary paths
addpath utility/ sh/ sh/rotation/ dswt/ experiments/ qmc/

% Obtain SH coefficients, either by projecting a signal into SH or by 
% choosing random coefficients

% Compute SH coefficients (slow, hence only applicable for very low bands)
% disp(sprintf('Projecting test signal into Spherical Harmonics ...'));
% shRunSimpleExperiment;
% disp(sprintf('Finished projecting test signal into Spherical Harmonics.'));
% Not all methods currently support multiple data channels
% coeffs = coeffs(1,:);

% Generate SH coefficeints 
max_band = 25;
coeffs = rand( 1, (max_band+1)^2);

% Variables for rotation experiment
% Data path  for rotation matrices
data_path = 'data/';

% Oversampling factor
os_factor = 2.0;

% Method for generating sampling points on the sphere
vert_method = 'halton';

% Precompute change of basis matrices
t_precomp = shFindCbm( max_band, os_factor, data_path, vert_method);

% create a random rotation
Rs = generateRandomRotations( 200);
R = Rs{95};
angles = zeros( 1,3);
[angles(1), angles(2), angles(3)] = getRotationEulerZYZMatrix( R);

%% Compute rotation using different algorithms

% compute rotation using RKHS method
[coeffs_r_rkhs, t_rkhs] = shRKHSRotationR( max_band, coeffs, ...
                                           os_factor, R, ...
                                           data_path, vert_method);

% Compute rotation using Simon's implementation based on the algorithm by
% Blanco
[coeffs_r_simons, t_simons] = shRotateSimons( max_band, coeffs, ...
                                         angles(1), angles(2), angles(3));

% The original algorithm by Gimbutas and Greengards
coeffs_r_multipole = shRotateMultipole( max_band, coeffs, ...
                                        angles(1), angles(2), angles(3));


% The algorithm by Gimbutas and Greengards but used for arbitrary rotations
% without separate rotation steps around the Z axis
coeffs_r_spectral = shRotateSpectralEquator( max_band, coeffs, ...
                                       angles(1), angles(2), angles(3));   
                                     
% Spectral rotation algoritm for equi-latitude sample points off the
% equator
coeffs_r_rkhs_s_1 = shRotateRKHSSpectral( max_band, coeffs, ...
                                        angles(1), angles(2), angles(3));                                     

%% Post-process results

disp(sprintf( 'Maximum SH band: %i', max_band));              
disp(sprintf('Execution time RKHS / Simons : %.7g s / %.7g s', ...
              t_rkhs, t_simons));             
disp(sprintf('Precomputation time : %.7g s', t_precomp));

% compute per band error (use Simons as reference solution)
errors = zeros(5,max_band);
index = 2;
for( band = 1 : max_band)
  
  band_size = 2*band+1;
  
  idx = [index : index+band_size-1];
  
  errors(1,band) = norm( coeffs_r_simons(idx) - coeffs_r_rkhs(idx)) ...
                / (norm(coeffs_r_simons(idx)) * norm(coeffs_r_rkhs(idx)));
  
  errors(2,band) = norm( coeffs_r_simons(idx) - coeffs_r_multipole(idx))...
           / (norm(coeffs_r_simons(idx)) * norm(coeffs_r_multipole(idx)));
  
	errors(3,band) = norm( coeffs_r_simons(idx) - coeffs_r_spectral(idx))...
           / (norm(coeffs_r_simons(idx)) * norm(coeffs_r_spectral(idx)));
  
	errors(4,band) = norm( coeffs_r_simons(idx) - coeffs_r_rkhs_s_1(idx))...
           / (norm(coeffs_r_simons(idx)) * norm(coeffs_r_rkhs_s_1(idx))); 
         
	errors(5,band) = norm( coeffs_r_simons(idx) - coeffs_r_rkhs_s_2(idx))...
           / (norm(coeffs_r_simons(idx)) * norm(coeffs_r_rkhs_s_2(idx))); 
                  
  index = index + band_size;
              
end

% finally plot
figure;
plot( 1:max_band, errors(1,:), 'r', ...
      1:max_band, errors(2,:), 'b', ...
      1:max_band, errors(3,:), 'g', ...
      1:max_band, errors(4,:), 'm', ...
      1:max_band, errors(5,:), 'k');    
set( gca, 'YScale', 'log');    
legend( 'RKHS rotation', ...
        'Spectral (original)', ...
        'Spectral (arbitrary rotations)', ...
        'RKHS Spectral (equi-latitude)', ...
        'RKHS Spectral (equi-latitude, with fix)', ...
        'Location', 'NorthWest');