function [G, N, E] = dimSlepianCapSymm( L, Theta, epsilon)
%
% [G, N, E] = dimSlepianCapSymm( L, Theta, epsilon)
%
% Compute approximate dimension of spatio-spectral concentration problem
% when the region of concentration is a spherical cap with theta < Theta
% and the signal (or tapers) is assumed to be radially symmetric. This
% situation is discussed in detail in Wieczorek & Simons (2005).
%
% @note The present formula is only a conjecture and in particular the
%       boundary term is not known exactly.
%
% @return G        Generalized Shannon number, approximate dimension
% @return N        Shannon number, number of eigenvalues very close to unity
% @return E        Number of eigenvalues in the region of the spectrum with
%                  exponential decay from 1 to 0.
% @param  L        Maximum Spherical Harmonics band
% @param  Theta    Maximum latitude of the spherical cap (in degrees)
% @param  epsilon  Epsilon threhold for the approximate dimension, that is
%                  G determines the number of eigenvalues larger than
%                  epsilon.

  % default input arguments
  defval( 'epsilon', 0.01);

  Theta = deg_to_rad(Theta);

  % Shannon number
  N = (L+1) * (Theta/pi);

  % Second term necessary for small regions of spatio-spectral
  % concentration
  [A,dA] = areaSphericalCap( Theta);
  E = log10((1-0.01)/0.01) * log10(N) *  log10( L+1 * dA) + 6/(L+1);
  
  % Generalized Shannon number
  G = N + E;

end