function experimentTransport( max_band, op_type, Theta_degs, ...
                              rho, samples_cap )
%
% experimentTransport( max_band, op_type, Theta_degs, rho, samples_cap)
%
% Run experiment comparing the spectrum of a numerical Phong-like transport 
% operator and the spectrum of the corresponding spatio-spectral
% concentration problem on the sphere.
%
% @param max_band     max. Spherical Harmonics band L 
%                     (def = 10)
% @param op_type      transport operator type ('diffuse', 'phong')
%                     (def = 'phong')
% @param Theta_degs   max. latitude(s) for Spherical caps 
%                     (def = [15, 20, 25, 30])
% @param rho          exponent for the transport operator (supported for
%                     phong *and* diffuse transport)
%                     (def = '64')
% @param samples_cap  number of samples in the cap
%                     (def = 2000)
  
  %% Parameters

  % Max. latitude(s) for spherical caps
  defval( 'max_band', 10);
  defval( 'op_type',  'phong');
  if( strcmp( 'phong', op_type))
    defval( 'Theta_degs', [5, 10, 15, 20]);
    defval( 'rho', 128);
  elseif( strcmp( 'diffuse', op_type))
    defval( 'Theta_degs', [15, 20, 25, 30]);
    defval( 'rho', 1);
  else
    error( 'Unsupported argument', 'Unsupported argument');
  end
  defval( 'samples_cap', 2000);
  
  % Spherical Harmonics parameters
  sqrt_samples_sh_proj = 100;

  % transport operators
  Ts = {};
  
  %% Compute all transport operators
  for( k = 1 : numel(Theta_degs))

    % Store current transport operator
    if( strcmp( 'phong', op_type))
      % Store current transport operator
      Ts{k} = transportOperatorPhong( max_band, Theta_degs(k), rho, ...
                                    samples_cap, sqrt_samples_sh_proj);
    elseif( strcmp( 'diffuse', op_type))
      Ts{k} = transportOperatorDiffuse( max_band, Theta_degs(k), rho, ...
                                        samples_cap, sqrt_samples_sh_proj);
    end
       
  end

  %% Plotting etc.

  gen_shannon_nums = zeros(1,numel(Theta_degs));
  shannon_nums = zeros(1,numel(Theta_degs));
  evals_max = zeros(1,numel(Theta_degs));
  legend_str = {};

  colors = {'r', 'g', 'b', 'k', 'm', 'c'};

  figure;
  hold on;

  xlim([1, (max_band+1)^2]);
  set( gca, 'XScale', 'log');

  for( k = 1 : numel(Theta_degs))

     % Compute Slepian spectrum for spherical cap
     [G,V,EL,EM,N,GM2AL,MTAP,IMTAP] = glmalpha( 2*Theta_degs(k), max_band, 1);

     % Sort eigenvalues
     ev = sort(V, 'descend');
     plot( 1:numel(ev), ev, ['-x', colors{k}]);
     evals_max(k) = ev(1);

     % legend string
     legend_str{k} = ['\Theta = ', num2str(Theta_degs(k)), '\circ'];

     % Compute estimate for dimension 
     [GN, N] = dimSlepianCap( max_band, 2*Theta_degs(k));
     disp( sprintf( 'Theta = %i: GN = %f , N = %f', Theta_degs(k), GN, N));
     gen_shannon_nums(k) = GN;
     shannon_nums(k) = N;

  end

  % plot legend (now so that there is only one entry for every Theta value)
  legend( legend_str);

  for( k = 1 : numel(Theta_degs))

     % compute SVD of transport operator
     [U,S,V] = svd(Ts{k});
     svals = diag(S);

     scaling = evals_max(k) / svals(1);
     plot( 1:numel(svals), svals * scaling, ['--x', colors{k}]);

  end

  % plot vertical lines
  for( i = 1 : numel(Theta_degs))
    vline( gen_shannon_nums(i), ['-.', colors{i}]);
    vline( shannon_nums(i), [':', colors{i}]);
  end
  
  set( gca, 'YAxisLocation', 'left');
  
end