function evals = plotEvalsSlepianCap( L, Theta, epsilon, x_scale)
%
% plotEvalsSlepianCap( L, Theta, epsilon, x_scale)
%
% Plot spectrum of spatio-spectral concentration problem for spherical cap
% with \theta \leq \Theta and for bandlimit L. Plotted are also the Shannon
% and generalized Shannon numbers (with respect to epsilon).
%
% @return
% @param  L        Spherical Harmonics band limit.
% @param  Theta    Maximum latitude of the spherical cap (can be a list of 
%                  Theta values in which case all are plotted in one fig.
% @param  epsilon  Threshold for Shannon and generalized Shannon number.

  defval( 'L', 20)
  defval( 'Theta', [ 5, 10, 20, 30, 40, 50]);
  defval( 'epsilon', 0.01);
  defval( 'x_scale', 'log');

  % plotting
  colors = {'r', 'g', 'b', 'k', 'm', 'c'};
  figure;
  hold on;
  
  xlim([1, (L+1)^2]);
  set( gca, 'XScale', x_scale);
  
  % plot vertical lines a posterior since the max Y values changes in the
  % cours of the iterations
  gen_shannon_nums = zeros( 1, numel(Theta));
  shannon_nums = zeros( 1, numel(Theta));
  evals = zeros( numel(Theta), (L+1)^2);
  
  for( i = 1 : numel(Theta))
    
    % Compute spectrum for spherical cap
    [G,V,EL,EM,N,GM2AL,MTAP,IMTAP] = glmalpha( Theta(i), L, 1);

    % Sort eigenvalues
    ev = sort(V, 'descend');
    plot( 1:numel(ev), ev, ['-x', colors{i}]);
    evals(i,:) = ev;
   
    % Compute estimate for dimension 
    [GN, N] = dimSlepianCap( L, Theta(i), 0.01);
    gen_shannon_nums(i) = GN;
    shannon_nums(i) = max(1,N);
     
    legend_str{i} = ['\Theta = ', num2str(Theta(i))];
    
  end

  % plot vertical lines
  for( i = 1 : numel(Theta))
    vline( gen_shannon_nums(i), ['-.', colors{i}]);
    %vline( shannon_nums(i), [':', colors{i}]);
  end
  
  % plot legend etc.
  % title( sprintf( 'Band %i', L));
  legend( legend_str);
  
  title( ['L = ', num2str(L)]);
  
end