function T = transportOperatorPhong( max_band, Theta, rho, samples_cap, ...
                                      sqrt_samples_sh_proj)
%
% T = transportOperatorPhong( max_band, Theta, rho, samples_cap, ...
%                             sqrt_samples_sh_proj)
%
% Compute a numerical transport operator for a Phong-like kernel 
% T = (n(x) \cdot \omega)^{\rho} over a spherical cap with theta \leq Theta
% using samples_cap samples which are randomly distributed over the spherical 
% cap. Each row of T corresponds to the transport operator for one sample
% point, the first entry is the sample at the north pole. 
%
% @return T  transport operator 
% @param  max_band    maximum Spherical Harmonics band
% @param  Theta        maximum latitude of spherical cap
% @param  rho          Phong exponent
% @param  samples_cap  number of samples on the spherical cap
% @param  sqrt_samples_sh_proj  number of samples for Spherical Harmonics 
%                               projection (default = 100) 

  % computed transport operators are stored, first try to load and compute
  % it only if this fails

  % file name for the stored transport operator
  base_dir = './data';
  fname = [base_dir, '/', 'transport_phong'];
  fname = [fname, '_L', num2str(max_band)];
  fname = [fname, '_Theta', num2str(Theta)];
  fname = [fname, '_rho', num2str( rho)];
  fname = [fname, '_samples', num2str( samples_cap)];
  
  try
  
    % try to load
    load( fname);
    disp(sprintf('Loaded transport operator.'));
  
  catch
    
    disp(sprintf('Computing transport operator (slow).'));
    defval( 'sqrt_samples_sh_proj', 100);

    % transport operator
    T = zeros( samples_cap, (max_band+1)^2);

    % Generate random points in spherical cap
    omega = uniformPointsS2( samples_cap, 'cap', deg_to_rad(Theta));
    omega_r3 = s2_to_r3_list( omega);

    r3 = zeros( size(omega_r3));
   
    % samples for computing SH coefficients
    samples = shGenerateSamples( sqrt_samples_sh_proj, max_band);
    
    % for all points
    for( i = 1 : size( omega,1))

      % compute reflection direction
      
      % compute rotation for current sample point
      rot_axis = cross( [0,0,1], omega_r3(i,:));
      rot_axis = rot_axis ./ norm(rot_axis);
      rot_angle = -acos( dot( [0,0,1], omega_r3(i,:)));
      R = getRotationMatrixAxisAngle( rot_axis, rad_to_deg(rot_angle));

      % compute direction of incoming lighting in the local coordinate
      % frame
      l_local = R * [0,0,1]';
      
      % reflect the l
      reflect_local = reflectVec( [0,0,1]', l_local);
      
      % rotate back to global coordinate frame
      reflect_global = R' * reflect_local;
      
      % project signal into Spherical Harmonics
      T(i,:) = shProjectPhong( samples, max_band, reflect_global, rho);
      
    end

    % store compute transport operator
    save( fname, 'T');
 
  end  % end catch
  
end